import { WorkingAreaComponent } from '../working-area.component';
import { GameMode } from './gameMode';
import * as PIXI from 'pixi.js';

/**
 * 多点连线
 */
export class MultipointIcon extends PIXI.Container {
    public pointsData: PIXI.Point[];
    public pointsGraphics: PIXI.Graphics[] = [];
    public iconsTilingSprite: PIXI.TilingSprite[] = [];
    style = new PIXI.TextStyle({
        fontFamily: 'Arial',
        fontSize: 18,
        fontStyle: 'normal',
        fontWeight: 'bold',
        fill: ['#000000'],
        stroke: '#ffffff',
        strokeThickness: 3,
        dropShadow: true,
        dropShadowColor: '#000000',
        dropShadowBlur: 3,
        dropShadowAngle: Math.PI / 6,
        dropShadowDistance: 1,
        wordWrap: false,
        wordWrapWidth: 100,
    });

    public text = new PIXI.Text(this.assetData.Name
        + '\r\n'
        + this.assetData.PropertyInfos?.find(item => item.PropertyName === '名称/编号')?.PropertyValue, this.style);
    /**
     *
     * @param texture 图片素材
     * @param points  点集合
     */
    constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
        super();
        this.name = this.assetData.Id;
        this.pointsData = this.assetData.MultiPoint;
        this.x = this.assetData.Point.x;
        this.y = this.assetData.Point.y;
        this.workingArea.backgroundImage.addChild(this);
        // 画线图标
        for (let i = 0, count = this.pointsData.length - 1; i < count; i++) {
            const pointA = this.pointsData[i];
            const pointB = this.pointsData[i + 1];

            const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
            const a = pointB.x - pointA.x;
            const b = pointB.y - pointA.y;
            const distance = Math.sqrt(a * a + b * b);

            const icon = new PIXI.TilingSprite(PIXI.Texture.from(this.assetData.ImageUrl), distance, 64);
            icon.anchor.set(0, 0.5);
            icon.x = pointA.x;
            icon.y = pointA.y;
            icon.angle = angle;
            this.iconsTilingSprite.push(icon);
            this.addChild(icon);
            if (i === 0) {
                this.text.anchor.set(0.5);
                this.text.position = icon.position;
                this.text.y -= this.assetData.Height;
                this.addChild(this.text);
            }
        }
        // 画点
        this.pointsData.forEach((item, index, array) => {
            const iconPoint = new PIXI.Graphics();
            iconPoint.lineStyle(1, 0xFFBD01, 1);
            iconPoint.beginFill(0xFFFFFF, 1);
            iconPoint.drawCircle(0, 0, 15);
            iconPoint.x = item.x;
            iconPoint.y = item.y;
            iconPoint.endFill();
            iconPoint.visible = false;
            this.pointsGraphics.push(iconPoint);
            this.addChild(iconPoint);
        });
        // 添加圆点事件
        this.pointsGraphics.forEach((item, index, array) => {
            item.interactive = true;
            item.on('mousedown', event => {
                event.stopPropagation();
                if (this.workingArea.allowEdit && this.assetData.GameMode === this.workingArea.canvasData.gameMode) {
                    event.currentTarget.data = event.data;
                    event.currentTarget.alpha = 0.5;
                    event.currentTarget.dragging = true;
                }
            })
                .on('mouseup', event => {
                    if (event.currentTarget.dragging) {
                        event.currentTarget.alpha = 1;
                        event.currentTarget.dragging = false;
                        event.currentTarget.data = null;
                    }
                })
                .on('mouseupoutside', event => {
                    if (event.currentTarget.dragging) {
                        event.currentTarget.alpha = 1;
                        event.currentTarget.dragging = false;
                        event.currentTarget.data = null;
                    }
                })
                .on('mousemove', event => {
                    if (event.currentTarget.dragging) {
                        const newPosition = event.currentTarget.data.getLocalPosition(event.currentTarget.parent);
                        event.currentTarget.x = newPosition.x;
                        event.currentTarget.y = newPosition.y;

                        this.assetData.MultiPoint[index].x = newPosition.x;
                        this.assetData.MultiPoint[index].y = newPosition.y;
                        this.workingArea.canvasData.isChange = true;

                        if (index === 0) {// 第一个点
                            this.iconsTilingSprite[index].x = newPosition.x;
                            this.iconsTilingSprite[index].y = newPosition.y;

                            const pointA = array[index];
                            const pointB = array[index + 1];

                            const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
                            const a = pointB.x - pointA.x;
                            const b = pointB.y - pointA.y;
                            const distance = Math.sqrt(a * a + b * b);
                            this.iconsTilingSprite[index].angle = angle;
                            this.iconsTilingSprite[index].width = distance;

                            this.text.position = this.iconsTilingSprite[index].position;
                            this.text.y -= this.assetData.Height;
                        } else if (index < array.length - 1) {// 不是第一个点，也不是最后一个点
                            this.iconsTilingSprite[index].x = newPosition.x;
                            this.iconsTilingSprite[index].y = newPosition.y;

                            const pointA = array[index]; // 当前点
                            const pointB = array[index + 1]; // 后一个点
                            const pointC = array[index - 1]; // 前一个点

                            const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI);
                            const a = pointB.x - pointA.x;
                            const b = pointB.y - pointA.y;
                            const distance = Math.sqrt(a * a + b * b);
                            this.iconsTilingSprite[index].angle = angle;
                            this.iconsTilingSprite[index].width = distance;

                            const angleC = Math.atan2((pointA.y - pointC.y), (pointA.x - pointC.x)) * (180 / Math.PI);
                            const aC = pointA.x - pointC.x;
                            const bC = pointA.y - pointC.y;
                            const distanceC = Math.sqrt(aC * aC + bC * bC);
                            this.iconsTilingSprite[index - 1].angle = angleC;
                            this.iconsTilingSprite[index - 1].width = distanceC;
                        } else if (index === array.length - 1) { // 最后一个点
                            const pointA = array[index]; // 当前点
                            const pointC = array[index - 1]; // 前一个点

                            const angleC = Math.atan2((pointA.y - pointC.y), (pointA.x - pointC.x)) * (180 / Math.PI);
                            const aC = pointA.x - pointC.x;
                            const bC = pointA.y - pointC.y;
                            const distanceC = Math.sqrt(aC * aC + bC * bC);
                            this.iconsTilingSprite[index - 1].angle = angleC;
                            this.iconsTilingSprite[index - 1].width = distanceC;
                        }
                    }
                })
                .on('rightclick', event => {
                })
                .on('mouseover', event => {

                });
        });
        // // 缩放
        // this.workingArea.on('backgroundScale', data => {
        //   const scale = 1 / data;
        //   this.text.scale.set(scale);
        // });
        // 添加选中事件
        this.iconsTilingSprite.forEach((item, index, array) => {
            item.interactive = true;
            item.on('mousedown', event => {
                event.stopPropagation();
                this.workingArea.selection.selectOne(this);
                if (this.workingArea.allowEdit && this.assetData.GameMode === this.workingArea.canvasData.gameMode) {
                    event.currentTarget.parent.data = event.data;
                    event.currentTarget.parent.alpha = 0.5;
                    event.currentTarget.parent.dragging = true;

                    event.currentTarget.parent.dragPoint = event.data.getLocalPosition(event.currentTarget.parent.parent);
                    event.currentTarget.parent.dragPoint.x -= event.currentTarget.parent.x;
                    event.currentTarget.parent.dragPoint.y -= event.currentTarget.parent.y;
                }
            })
                .on('mouseup', event => {
                    if (event.currentTarget.parent.dragging) {
                        event.currentTarget.parent.alpha = 1;
                        event.currentTarget.parent.dragging = false;
                        event.currentTarget.parent.data = null;
                    }
                })
                .on('mouseupoutside', event => {
                    if (event.currentTarget.parent.dragging) {
                        event.currentTarget.parent.alpha = 1;
                        event.currentTarget.parent.dragging = false;
                        event.currentTarget.parent.data = null;
                    }
                })
                .on('mousemove', event => {
                    if (event.currentTarget.parent.dragging) {
                        const newPosition = event.currentTarget.parent.data.getLocalPosition(event.currentTarget.parent.parent);
                        event.currentTarget.parent.x = newPosition.x - event.currentTarget.parent.dragPoint.x;
                        event.currentTarget.parent.y = newPosition.y - event.currentTarget.parent.dragPoint.y;

                        this.assetData.Point = new PIXI.Point(this.x, this.y);
                        this.workingArea.canvasData.isChange = true;
                    }
                })
                .on('rightclick', event => {

                });
        });
    }
    /**
     * 设置点显示状态
     * @param value 显示状态
     */
    public setPointVisiable(value: boolean) {
        this.pointsGraphics.forEach((item) => {
            item.visible = value;
        });
    }
    // 设置名称
    public setNameVisible(value: boolean, mode: GameMode) {
        if (this.assetData.GameMode === mode) {
            this.text.visible = value;
        }
    }
    //
    public refresh() {
        this.text.text = this.assetData.Name
            + '\r\n'
            + this.assetData.PropertyInfos.find(item => item.PropertyName === '名称/编号')?.PropertyValue;
    }
}