import { WorkingAreaComponent } from '../working-area.component';
import * as PIXI from 'pixi.js';

/**
 * 管线
 */
export class Pipeline extends PIXI.Container {
    public line: PIXI.Graphics = new PIXI.Graphics();
    constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
        super();
        this.name = this.assetData.Id;
        this.x = this.assetData.Point.x;
        this.y = this.assetData.Point.y;
        this.workingArea.backgroundImage.addChild(this);
        this.addChild(this.line);
        // 画线图标
        this.refresh();
        this.interactive = true;
        this.on('mousedown', event => {
            console.log(this.assetData);
            event.stopPropagation();
            this.workingArea.selection.selectOne(this);
        });
    }
    /**
     * 刷新
     */
    public refresh() {
        this.line.clear();
        this.line.lineStyle(5, 0x0000ff, 1);
        for (let i = 0, count = this.assetData.MultiPoint.length; i < count; i++) {
            const pointA = this.assetData.MultiPoint[i];
            if (i === 0) {
                this.line.moveTo(pointA.x, pointA.y);
            } else {
                this.line.lineTo(pointA.x, pointA.y);
                if (i === count - 1) {
                    const pointB = this.assetData.MultiPoint[i - 1];
                    const angle = Math.atan2((pointA.y - pointB.y), (pointA.x - pointB.x)) * (180 / Math.PI) + 90;
                    this.line.beginFill(0x0000ff);
                    this.line.drawStar(pointA.x, pointA.y, 3, 5, 0, (Math.PI / 180 * angle));
                } else {
                    this.line.drawCircle(pointA.x, pointA.y, 5);
                }
            }
        }
        this.line.endFill();
        // if (this.assetData.MultiPoint.length >= 2) {
        //   const pointA = this.assetData.MultiPoint[this.assetData.MultiPoint.length - 2];
        //   const pointB = this.assetData.MultiPoint[this.assetData.MultiPoint.length - 1];
        //   const angle = Math.atan2((pointB.y - pointA.y), (pointB.x - pointA.x)) * (180 / Math.PI) + 90;
        //   this.beginFill(0x0000ff);
        //   this.drawStar(pointB.x, pointB.y, 3, 5, 0, (Math.PI / 180 * angle));
        //   this.endFill();
        // }
    }
}