import { WorkingAreaComponent } from '../working-area.component';
import * as ObjectID from 'bson-objectid';
import { GameMode } from './gameMode';
import { Pipeline } from './pipeline';
import { PaintMode } from './paintModel';
import * as PIXI from 'pixi.js';
import { throwMatDialogContentAlreadyAttachedError } from '@angular/material/dialog';

/**
 * 单点图标
 */
export class SinglePointIcon extends PIXI.Container {
    style = new PIXI.TextStyle({
        fontFamily: 'Arial',
        fontSize: 18,
        fontStyle: 'normal',
        fontWeight: 'bold',
        fill: ['#000000'],
        stroke: '#ffffff',
        strokeThickness: 3,
        dropShadow: true,
        dropShadowColor: '#000000',
        dropShadowBlur: 3,
        dropShadowAngle: Math.PI / 6,
        dropShadowDistance: 1,
        wordWrap: false,
        wordWrapWidth: 100,
    });

    public text = new PIXI.Text(this.assetData.Name
        + '\r\n'
        + this.assetData.PropertyInfos?.find(item => item.PropertyName === '名称/编号')?.PropertyValue, this.style);

    /**
     * 选中圆点
     */
    public selectedPointTexture = PIXI.Texture.from('assets/images/handle-main.png');
    private image = PIXI.Sprite.from(this.assetData.ImageUrl);
    public graphics = new PIXI.Graphics();
    public connectionPoint = new PIXI.Graphics();

    // 显示连接点
    public showConnectionPoint = false;

    constructor(public assetData: any, private workingArea: WorkingAreaComponent) {
        super();
        this.workingArea.backgroundImage.addChild(this);
        this.x = this.assetData.Point.x;
        this.y = this.assetData.Point.y;
        this.name = this.assetData.Id;

        this.image.angle = this.assetData.Angle;

        this.image.x = 0;
        this.image.y = 0;
        this.image.width = this.assetData.Width;
        this.image.height = this.assetData.Height;

        this.image.alpha = 1;
        this.image.anchor.set(0.5);
        this.image.interactive = true;
        this.image
            .on('mousedown', event => {
                console.log(this.workingArea.canvasData.selectPanelPoint);
                event.stopPropagation();
                this.workingArea.selection.selectOne(this);
                if (this.assetData.CanConnection && this.parent === this.workingArea.backgroundImage) {
                    if (this.workingArea.getPaintMode() === PaintMode.Pipeline) {
                        if (this.workingArea.paintingPipeline === null) {
                            this.workingArea.previewLineSegment.visible = true;
                            this.workingArea.currentClickPoint.position =
                                new PIXI.Point(this.workingArea.circleShadow.x, this.workingArea.circleShadow.y);
                            this.workingArea.paintPoints.push(new PIXI.Point(this.position.x, this.position.y));
                            const tempData = {
                                Id: ObjectID.default.generate(),
                                MultiPoint: JSON.parse(JSON.stringify(this.workingArea.paintPoints)),
                                Point: new PIXI.Point(0, 0),
                                Name: '管线',
                                LinkedObjects: [],
                            };
                            this.workingArea.paintingPipeline = new Pipeline(tempData, this.workingArea);
                            this.workingArea.paintingPipeline.assetData.LinkedObjects.push(this);
                            this.assetData.Pipelines.push(this.workingArea.paintingPipeline);
                        } else {
                            this.workingArea.previewLineSegment.visible = false;
                            this.workingArea.currentClickPoint.position =
                                new PIXI.Point(this.workingArea.circleShadow.x, this.workingArea.circleShadow.y);
                            this.workingArea.paintPoints.push(new PIXI.Point(this.position.x, this.position.y));
                            this.workingArea.paintingPipeline.assetData.MultiPoint =
                                JSON.parse(JSON.stringify(this.workingArea.paintPoints));
                            this.workingArea.paintingPipeline.assetData.LinkedObjects.push(this);
                            this.assetData.Pipelines.push(this.workingArea.paintingPipeline);
                            this.workingArea.paintingPipeline.refresh();
                            this.workingArea.paintingPipelineFinish();
                        }
                    }
                }
                // 如果链接对象不为空,禁止移动
                if (this.workingArea.allowEdit && this.assetData.GameMode === this.workingArea.canvasData.gameMode) {
                    event.currentTarget.parent.data = event.data;
                    event.currentTarget.parent.alpha = 0.5;
                    event.currentTarget.parent.dragging = true;
                }
            })
            .on('mouseup', event => {
                if (event.currentTarget.parent.dragging) {
                    event.currentTarget.parent.alpha = 1;
                    event.currentTarget.parent.dragging = false;
                    event.currentTarget.parent.data = null;
                }
            })
            .on('mouseupoutside', event => {
                if (event.currentTarget.parent.dragging) {
                    event.currentTarget.parent.alpha = 1;
                    event.currentTarget.parent.dragging = false;
                    event.currentTarget.parent.data = null;
                }
            })
            .on('mousemove', event => {
                if (event.currentTarget.parent.dragging) {
                    // 如果拖动过程中发现父对象不是背景图
                    if (this.parent !== this.workingArea.backgroundImage) {
                        this.setParent(this.workingArea.backgroundImage);
                        if (this.assetData.FixedSize) {
                            const scale = 1 / this.workingArea.backgroundImage.scale.x;
                            this.scale.set(scale);
                        }
                    }
                    const newPosition = event.currentTarget.parent.data.getLocalPosition(event.currentTarget.parent.parent);
                    event.currentTarget.parent.x = newPosition.x;
                    event.currentTarget.parent.y = newPosition.y;
                    this.assetData.Point = new PIXI.Point(this.x, this.y);
                    this.workingArea.canvasData.isChange = true;
                }
            })
            .on('rightclick', event => {

            })
            .on('mouseover', event => {
                // this.connectionPoint.visible = true;
                this.setSelectionBox(true);
            })
            .on('mouseout', event => {
                // this.connectionPoint.visible = false;
                this.setSelectionBox(false);
            });
        this.text.x = this.image.x;
        this.text.y = this.image.y - this.image.height / 2;
        this.text.anchor.set(0.5, 1);

        if (this.assetData.GameMode === 2) {
            this.text.visible = false;
        }

        this.connectionPoint.beginFill();
        this.connectionPoint.drawCircle(0, 0, 5);
        this.connectionPoint.endFill();
        this.connectionPoint.visible = true;
        this.connectionPoint.interactive = true;
        this.connectionPoint
            .on('mouseover', event => {
                console.log('b');
                this.graphics.lineStyle(2, 0x00EB00, 1);
                this.graphics.drawRect(- this.connectionPoint.width / 2, - this.connectionPoint.width / 2,
                    this.connectionPoint.width, this.connectionPoint.width);
            })
            .on('mouseout', event => {
                this.graphics.clear();
            });
        this.addChild(this.text);
        this.addChild(this.image);
        this.addChild(this.graphics);
        // this.addChild(this.connectionPoint);

        // // up
        // const up = new PIXI.Sprite(this.selectedPointTexture);
        // up.anchor.set(0.5);
        // up.x = this.image.x;
        // up.y = this.image.y - (this.image.height / 2);
        // this.addChild(up);
        // // up-left
        // const upLeft = new PIXI.Sprite(this.selectedPointTexture);
        // upLeft.anchor.set(0.5);
        // upLeft.x = this.image.x - (this.image.width / 2);
        // upLeft.y = this.image.y - (this.image.height / 2);
        // this.addChild(upLeft);
        // // up-right
        // const upRight = new PIXI.Sprite(this.selectedPointTexture);
        // upRight.anchor.set(0.5);
        // upRight.x = this.image.x + (this.image.width / 2);
        // upRight.y = this.image.y - (this.image.height / 2);
        // this.addChild(upRight);
        // // down
        // const down = new PIXI.Sprite(this.selectedPointTexture);
        // down.anchor.set(0.5);
        // down.x = this.image.x;
        // down.y = this.image.y + (this.image.height / 2);
        // this.addChild(down);
        // // down-left
        // const downLeft = new PIXI.Sprite(this.selectedPointTexture);
        // downLeft.anchor.set(0.5);
        // downLeft.x = this.image.x - (this.image.width / 2);
        // downLeft.y = this.image.y + (this.image.height / 2);
        // this.addChild(downLeft);
        // // down-right
        // const downRight = new PIXI.Sprite(this.selectedPointTexture);
        // downRight.anchor.set(0.5);
        // downRight.x = this.image.x + (this.image.width / 2);
        // downRight.y = this.image.y + (this.image.height / 2);
        // this.addChild(downRight);
        // // left
        // const left = new PIXI.Sprite(this.selectedPointTexture);
        // left.anchor.set(0.5);
        // left.x = this.image.x - (this.image.width / 2);
        // left.y = this.image.y;
        // this.addChild(left);
        // // right
        // const right = new PIXI.Sprite(this.selectedPointTexture);
        // right.anchor.set(0.5);
        // right.x = this.image.x + (this.image.width / 2);
        // right.y = this.image.y;
        // this.addChild(right);
    }
    // 设置选择框
    public setSelectionBox(b: boolean) {
        if (b) {
            this.graphics.lineStyle(2, 0x00EB00, 1);
            this.graphics.drawRect(- this.image.width / 2, - this.image.height / 2, this.image.width, this.image.height);
        } else {
            this.graphics.clear();
        }
    }
    // 设置名称
    public setNameVisible(value: boolean, mode: GameMode) {
        if (this.assetData.GameMode === mode) {
            this.text.visible = value;
        }
    }
    // 刷新
    public refresh() {
        this.image.width = this.assetData.Width;
        this.image.height = this.assetData.Height;
        this.image.angle = this.assetData.Angle;
        this.text.text = this.assetData.Name
            + '\r\n'
            + this.assetData.PropertyInfos?.find(item => item.PropertyName === '名称/编号')?.PropertyValue;
        this.text.x = this.image.x;
        this.text.y = this.image.y - this.image.height / 2;
    }
}
